// Auxiliary functions

char espera_activa (int espera) {
	// Esta función espera un rato o hasta que se pulse una tecla.
	// Si se pulsa una tecla, devuelve 0
	
	// Esta función sólo funciona en Spectrum (espera 10 segundos).
	// en CPC no hay una interrupción cada 20ms, asín que esto no
	// sirve "pa ná".
	
	char res = 1;
	int i;
	
	for (i = 0; i < espera && res; i ++) {
		#asm
			halt
		#endasm
		if (sp_GetKey ()) 
			res = 0;
	}
	
	return res;
}

void fade_out () {  
	int i;
	
	/* El segundo programa de más de 5 lineas en Z80 ASM de na_th_an */ 
	/* Optimizado por Metalbrain */ 
	/* Esto "oscurece" un paso la pantalla. Llamado 8 veces la dejará en negro */ 

	for (i = 0; i < 8; i ++) {
		#asm 
		  ld  e, 3            ; 3 tercios 
		  ld  hl, 22528       ; aquí empiezan los atributos 
		  halt                ; esperamos retrazo. 
		.fade_out_bucle 
		  ld  a, (hl)         ; nos traemos el atributo actual 
		
		  ld  d, a            ; tomar atributo 
		  and  7              ; aislar la tinta 
		  jr  z, ink_done     ; si vale 0, no se decrementa 
		  dec  a              ; decrementamos tinta 
		.ink_done 
		  ld  b, a            ; en b tenemos ahora la tinta ya procesada. 
		
		  ld  a, d            ; tomar atributo 
		  and  56             ; aislar el papel, sin modificar su posición en el byte 
		  jr  z, paper_done   ; si vale 0, no se decrementa 
		  sub 8               ; decrementamos papel restando 8 
		.paper_done 
		  ld  c, a            ; en c tenemos ahora el papel ya procesado. 
		  ld  a, d 
		  and  192            ; nos quedamos con bits 6 y 7 (BRIGHT y FLASH) 
		  or  c               ; añadimos paper 
		  or  b               ; e ink, con lo que recompuesto el atributo 
		  ld  (hl),a          ; lo escribimos, 
		  inc  l              ; e incrementamos el puntero. 
		  jr  nz, fade_out_bucle ; continuamos hasta acabar el tercio (cuando L valga 0) 
		  inc  h              ; siguiente tercio 
		  dec  e 
		  jr  nz, fade_out_bucle ; repetir las 3 veces 
		#endasm 
	}
}

void srand (unsigned int new_seed) {
	seed [0] = new_seed;	
}

unsigned char rand () {
	unsigned char res;
	
	#asm
		.rand16
			ld	hl, _seed
			ld	a, (hl)
			ld	e, a
			inc	hl
			ld	a, (hl)
			ld	d, a
			
			;; Ahora DE = [SEED]
						
			ld	a,	d
			ld	h,	e
			ld	l,	253
			or	a
			sbc	hl,	de
			sbc	a, 	0
			sbc	hl,	de
			ld	d, 	0
			sbc	a, 	d
			ld	e,	a
			sbc	hl,	de
			jr	nc,	nextrand
			inc	hl
		.nextrand
			ld	d,	h
			ld	e,	l
			ld	hl, _seed
			ld	a,	e
			ld	(hl), a
			inc	hl
			ld	a,	d
			ld	(hl), a
			
			;; Ahora [SEED] = HL
		
			ld 	hl, _asm_int
			ld	a,	e	
			ld	(hl), a
			inc	hl
			ld	a,	d
			ld	(hl), a
			
			;; Ahora [ASM_INT] = HL
	#endasm
	
	res = asm_int [0];

	return res;
}
/*
int sgn (int x) {
	if (x < 0)
		return -1;
	else if (x = 0)
		return 0;
	else
		return 1;
}
*/
unsigned int abs (int x) {
	if (x >= 0)
		return x;
	else
		return -x;
}

unsigned char attr (char x, char y) {
	char *res = (char *) (22528 + x + (y << 5));
	return (*res & 63);
}

void draw_score (unsigned int score) {
	sp_PrintAtInv (22, 3, 71, 246);
	sp_PrintAtInv (22, 4, 71, 246 + (score / 10000));
	sp_PrintAtInv (22, 5, 71, 246 + (score % 10000) / 1000);
	sp_PrintAtInv (22, 6, 71, 246 + (score % 1000) / 100);
	sp_PrintAtInv (22, 7, 71, 246 + (score % 100) / 10);
	sp_PrintAtInv (22, 8, 71, 246 + (score % 10));
}

void draw_lives (unsigned char lives) {
	sp_PrintAtInv (22, 11, 71, 246);
	sp_PrintAtInv (22, 12, 71, 246 + lives);	
}

void draw_life (unsigned char life) {
	sp_PrintAtInv (22, 19, 71, 246 + (life / 100));
	sp_PrintAtInv (22, 20, 71, 246 + (life % 100) / 10);
	sp_PrintAtInv (22, 21, 71, 246 + (life % 10));	
}

void helmet_resta_vida () {
	if (helmet.status) {
		if (helmet.life > 0) {
			helmet.life -= 5;
		} else {
			if (helmet.lives > 0) {
				helmet.life = 100;
				helmet.lives --;
				draw_lives (helmet.lives);
				helmet.status = 0;
				helmet.ct = 75;
				helmet.frame = helmet.dir + 4;
			}
		}
		draw_life (helmet.life);
	}
}

void unpack_RAM3 (unsigned int address) {
	asm_int [0] = address;
	
	#asm	
		di
		ld b, 3
		call SetRAMBank
		
		ld hl, (_asm_int)
		ld de, 16384
		call depack
		
		ld b, 0
		call SetRAMBank
		ei
	#endasm
}

